@@ -4,12 +4,12 @@
 
 //! `IMPL` Boxed wrapper implementation.
 
-use std::ops::{Deref, DerefMut};
-use std::fmt;
 use std::cmp;
+use std::fmt;
 use std::hash::{Hash, Hasher};
 use std::marker::PhantomData;
 use std::mem;
+use std::ops::{Deref, DerefMut};
 use std::ptr;
 use translate::*;
 
@@ -17,69 +17,38 @@ use translate::*;
 #[macro_export]
 macro_rules! glib_boxed_wrapper {
     ([$($attr:meta)*] $name:ident, $ffi_name:path, @copy $copy_arg:ident $copy_expr:expr,
-     @free $free_arg:ident $free_expr:expr,
+     @free $free_arg:ident $free_expr:expr, @init $init_arg:ident $init_expr:expr, @clear $clear_arg:ident $clear_expr:expr,
      @get_type $get_type_expr:expr) => {
-        glib_boxed_wrapper!([$($attr)*] $name, $ffi_name, @copy $copy_arg $copy_expr,
-            @free $free_arg $free_expr);
+        glib_boxed_wrapper!(@generic_impl [$($attr)*] $name, $ffi_name);
+        glib_boxed_wrapper!(@memory_manager_impl $name, $ffi_name, @copy $copy_arg $copy_expr, @free $free_arg $free_expr,
+                            @init $init_arg $init_expr, @clear $clear_arg $clear_expr);
+        glib_boxed_wrapper!(@value_impl $name, $ffi_name, @get_type $get_type_expr);
+    };
 
-        impl $crate::types::StaticType for $name {
-            fn static_type() -> $crate::types::Type {
-                #[allow(unused_unsafe)]
-                unsafe { $crate::translate::from_glib($get_type_expr) }
-            }
-        }
-
-        #[doc(hidden)]
-        impl<'a> $crate::value::FromValueOptional<'a> for $name {
-            unsafe fn from_value_optional(value: &$crate::Value) -> Option<Self> {
-                Option::<$name>::from_glib_full(gobject_ffi::g_value_dup_boxed(value.to_glib_none().0) as *mut $ffi_name)
-            }
-        }
-
-        #[doc(hidden)]
-        impl $crate::value::SetValue for $name {
-            unsafe fn set_value(value: &mut $crate::Value, this: &Self) {
-                gobject_ffi::g_value_set_boxed(value.to_glib_none_mut().0, $crate::translate::ToGlibPtr::<*const $ffi_name>::to_glib_none(this).0 as glib_ffi::gpointer)
-            }
-        }
-
-        #[doc(hidden)]
-        impl $crate::value::SetValueOptional for $name {
-            unsafe fn set_value_optional(value: &mut $crate::Value, this: Option<&Self>) {
-                gobject_ffi::g_value_set_boxed(value.to_glib_none_mut().0, $crate::translate::ToGlibPtr::<*const $ffi_name>::to_glib_none(&this).0 as glib_ffi::gpointer)
-            }
-        }
+    ([$($attr:meta)*] $name:ident, $ffi_name:path, @copy $copy_arg:ident $copy_expr:expr,
+     @free $free_arg:ident $free_expr:expr, @init $init_arg:ident $init_expr:expr, @clear $clear_arg:ident $clear_expr:expr) => {
+        glib_boxed_wrapper!(@generic_impl [$($attr)*] $name, $ffi_name);
+        glib_boxed_wrapper!(@memory_manager_impl $name, $ffi_name, @copy $copy_arg $copy_expr, @free $free_arg $free_expr,
+                            @init $init_arg $init_expr, @clear $clear_arg $clear_expr);
     };
 
     ([$($attr:meta)*] $name:ident, $ffi_name:path, @copy $copy_arg:ident $copy_expr:expr,
      @free $free_arg:ident $free_expr:expr) => {
+        glib_boxed_wrapper!(@generic_impl [$($attr)*] $name, $ffi_name);
+        glib_boxed_wrapper!(@memory_manager_impl $name, $ffi_name, @copy $copy_arg $copy_expr, @free $free_arg $free_expr);
+    };
+
+    ([$($attr:meta)*] $name:ident, $ffi_name:path, @copy $copy_arg:ident $copy_expr:expr,
+     @free $free_arg:ident $free_expr:expr, @get_type $get_type_expr:expr) => {
+        glib_boxed_wrapper!(@generic_impl [$($attr)*] $name, $ffi_name);
+        glib_boxed_wrapper!(@memory_manager_impl $name, $ffi_name, @copy $copy_arg $copy_expr, @free $free_arg $free_expr);
+        glib_boxed_wrapper!(@value_impl $name, $ffi_name, @get_type $get_type_expr);
+    };
+
+    (@generic_impl [$($attr:meta)*] $name:ident, $ffi_name:path) => {
         $(#[$attr])*
         #[derive(Clone)]
         pub struct $name($crate::boxed::Boxed<$ffi_name, MemoryManager>);
-
-        #[doc(hidden)]
-        pub struct MemoryManager;
-
-        impl $crate::boxed::BoxedMemoryManager<$ffi_name> for MemoryManager {
-            #[inline]
-            unsafe fn copy($copy_arg: *const $ffi_name) -> *mut $ffi_name {
-                $copy_expr
-            }
-
-            #[inline]
-            unsafe fn free($free_arg: *mut $ffi_name) {
-                $free_expr
-            }
-        }
-
-        #[doc(hidden)]
-        impl $crate::translate::Uninitialized for $name {
-            #[inline]
-            unsafe fn uninitialized() -> Self {
-                $name($crate::boxed::Boxed::uninitialized())
-            }
-        }
-
         #[doc(hidden)]
         impl $crate::translate::GlibPtrDefault for $name {
             type GlibType = *mut $ffi_name;
@@ -91,13 +60,13 @@ macro_rules! glib_boxed_wrapper {
 
             #[inline]
             fn to_glib_none(&'a self) -> $crate::translate::Stash<'a, *const $ffi_name, Self> {
-                let stash = self.0.to_glib_none();
+                let stash = $crate::translate::ToGlibPtr::to_glib_none(&self.0);
                 $crate::translate::Stash(stash.0, stash.1)
             }
 
             #[inline]
             fn to_glib_full(&self) -> *const $ffi_name {
-                (&self.0).to_glib_full()
+                $crate::translate::ToGlibPtr::to_glib_full(&self.0)
             }
         }
 
@@ -107,31 +76,31 @@ macro_rules! glib_boxed_wrapper {
 
             #[inline]
             fn to_glib_none_mut(&'a mut self) -> $crate::translate::StashMut<'a, *mut $ffi_name, Self> {
-                let stash = self.0.to_glib_none_mut();
+                let stash = $crate::translate::ToGlibPtrMut::to_glib_none_mut(&mut self.0);
                 $crate::translate::StashMut(stash.0, stash.1)
             }
         }
 
         #[doc(hidden)]
         impl<'a> $crate::translate::ToGlibContainerFromSlice<'a, *mut *const $ffi_name> for $name {
-            type Storage = (Vec<Stash<'a, *const $ffi_name, $name>>, Option<Vec<*const $ffi_name>>);
+            type Storage = (Vec<$crate::translate::Stash<'a, *const $ffi_name, $name>>, Option<Vec<*const $ffi_name>>);
 
             fn to_glib_none_from_slice(t: &'a [$name]) -> (*mut *const $ffi_name, Self::Storage) {
-                let v: Vec<_> = t.iter().map(|s| s.to_glib_none()).collect();
+                let v: Vec<_> = t.iter().map(|s| $crate::translate::ToGlibPtr::to_glib_none(s)).collect();
                 let mut v_ptr: Vec<_> = v.iter().map(|s| s.0).collect();
-                v_ptr.push(ptr::null_mut() as *const $ffi_name);
+                v_ptr.push(::std::ptr::null_mut() as *const $ffi_name);
 
                 (v_ptr.as_ptr() as *mut *const $ffi_name, (v, Some(v_ptr)))
             }
 
             fn to_glib_container_from_slice(t: &'a [$name]) -> (*mut *const $ffi_name, Self::Storage) {
-                let v: Vec<_> = t.iter().map(|s| s.to_glib_none()).collect();
+                let v: Vec<_> = t.iter().map(|s| $crate::translate::ToGlibPtr::to_glib_none(s)).collect();
 
                 let v_ptr = unsafe {
-                    let v_ptr = glib_ffi::g_malloc0(mem::size_of::<*const $ffi_name>() * (t.len() + 1)) as *mut *const $ffi_name;
+                    let v_ptr = $crate::glib_sys::g_malloc0(::std::mem::size_of::<*const $ffi_name>() * (t.len() + 1)) as *mut *const $ffi_name;
 
                     for (i, s) in v.iter().enumerate() {
-                        ptr::write(v_ptr.offset(i as isize), s.0);
+                        ::std::ptr::write(v_ptr.add(i), s.0);
                     }
 
                     v_ptr
@@ -142,10 +111,10 @@ macro_rules! glib_boxed_wrapper {
 
             fn to_glib_full_from_slice(t: &[$name]) -> *mut *const $ffi_name {
                 unsafe {
-                    let v_ptr = glib_ffi::g_malloc0(mem::size_of::<*const $ffi_name>() * (t.len() + 1)) as *mut *const $ffi_name;
+                    let v_ptr = $crate::glib_sys::g_malloc0(::std::mem::size_of::<*const $ffi_name>() * (t.len() + 1)) as *mut *const $ffi_name;
 
                     for (i, s) in t.iter().enumerate() {
-                        ptr::write(v_ptr.offset(i as isize), s.to_glib_full());
+                        ::std::ptr::write(v_ptr.add(i), $crate::translate::ToGlibPtr::to_glib_full(s));
                     }
 
                     v_ptr
@@ -155,7 +124,7 @@ macro_rules! glib_boxed_wrapper {
 
         #[doc(hidden)]
         impl<'a> $crate::translate::ToGlibContainerFromSlice<'a, *const *const $ffi_name> for $name {
-            type Storage = (Vec<Stash<'a, *const $ffi_name, $name>>, Option<Vec<*const $ffi_name>>);
+            type Storage = (Vec<$crate::translate::Stash<'a, *const $ffi_name, $name>>, Option<Vec<*const $ffi_name>>);
 
             fn to_glib_none_from_slice(t: &'a [$name]) -> (*const *const $ffi_name, Self::Storage) {
                 let (ptr, stash) = $crate::translate::ToGlibContainerFromSlice::<'a, *mut *const $ffi_name>::to_glib_none_from_slice(t);
@@ -205,6 +174,14 @@ macro_rules! glib_boxed_wrapper {
             }
         }
 
+        #[doc(hidden)]
+        impl $crate::translate::FromGlibPtrBorrow<*const $ffi_name> for $name {
+            #[inline]
+            unsafe fn from_glib_borrow(ptr: *const $ffi_name) -> Self {
+                $crate::translate::from_glib_borrow(ptr as *mut $ffi_name)
+            }
+        }
+
         #[doc(hidden)]
         impl $crate::translate::FromGlibContainerAsVec<*mut $ffi_name, *mut *mut $ffi_name> for $name {
             unsafe fn from_glib_none_num_as_vec(ptr: *mut *mut $ffi_name, num: usize) -> Vec<Self> {
@@ -214,14 +191,14 @@ macro_rules! glib_boxed_wrapper {
 
                 let mut res = Vec::with_capacity(num);
                 for i in 0..num {
-                    res.push($crate::translate::from_glib_none(ptr::read(ptr.offset(i as isize))));
+                    res.push($crate::translate::from_glib_none(::std::ptr::read(ptr.add(i))));
                 }
                 res
             }
 
             unsafe fn from_glib_container_num_as_vec(ptr: *mut *mut $ffi_name, num: usize) -> Vec<Self> {
                 let res = $crate::translate::FromGlibContainerAsVec::from_glib_none_num_as_vec(ptr, num);
-                glib_ffi::g_free(ptr as *mut _);
+                $crate::glib_sys::g_free(ptr as *mut _);
                 res
             }
 
@@ -232,9 +209,9 @@ macro_rules! glib_boxed_wrapper {
 
                 let mut res = Vec::with_capacity(num);
                 for i in 0..num {
-                    res.push($crate::translate::from_glib_full(ptr::read(ptr.offset(i as isize))));
+                    res.push($crate::translate::from_glib_full(::std::ptr::read(ptr.add(i))));
                 }
-                glib_ffi::g_free(ptr as *mut _);
+                $crate::glib_sys::g_free(ptr as *mut _);
                 res
             }
         }
@@ -253,7 +230,100 @@ macro_rules! glib_boxed_wrapper {
                 $crate::translate::FromGlibContainerAsVec::from_glib_full_num_as_vec(ptr, $crate::translate::c_ptr_array_len(ptr))
             }
         }
-    }
+    };
+
+    (@value_impl $name:ident, $ffi_name:path, @get_type $get_type_expr:expr) => {
+        impl $crate::types::StaticType for $name {
+            fn static_type() -> $crate::types::Type {
+                #[allow(unused_unsafe)]
+                unsafe { $crate::translate::from_glib($get_type_expr) }
+            }
+        }
+
+        #[doc(hidden)]
+        impl<'a> $crate::value::FromValueOptional<'a> for $name {
+            unsafe fn from_value_optional(value: &$crate::Value) -> Option<Self> {
+                $crate::translate::from_glib_full($crate::gobject_sys::g_value_dup_boxed($crate::translate::ToGlibPtr::to_glib_none(value).0) as *mut $ffi_name)
+            }
+        }
+
+        #[doc(hidden)]
+        impl $crate::value::SetValue for $name {
+            unsafe fn set_value(value: &mut $crate::Value, this: &Self) {
+                $crate::gobject_sys::g_value_set_boxed($crate::translate::ToGlibPtrMut::to_glib_none_mut(value).0, $crate::translate::ToGlibPtr::<*const $ffi_name>::to_glib_none(this).0 as $crate::glib_sys::gpointer)
+            }
+        }
+
+        #[doc(hidden)]
+        impl $crate::value::SetValueOptional for $name {
+            unsafe fn set_value_optional(value: &mut $crate::Value, this: Option<&Self>) {
+                $crate::gobject_sys::g_value_set_boxed($crate::translate::ToGlibPtrMut::to_glib_none_mut(value).0, $crate::translate::ToGlibPtr::<*const $ffi_name>::to_glib_none(&this).0 as $crate::glib_sys::gpointer)
+            }
+        }
+    };
+
+    (@memory_manager_impl $name:ident, $ffi_name:path, @copy $copy_arg:ident $copy_expr:expr, @free $free_arg:ident $free_expr:expr) => {
+        #[doc(hidden)]
+        pub struct MemoryManager;
+
+        impl $crate::boxed::BoxedMemoryManager<$ffi_name> for MemoryManager {
+            #[inline]
+            unsafe fn copy($copy_arg: *const $ffi_name) -> *mut $ffi_name {
+                $copy_expr
+            }
+
+            #[inline]
+            unsafe fn free($free_arg: *mut $ffi_name) {
+                $free_expr
+            }
+
+            #[inline]
+            unsafe fn init(_: *mut $ffi_name) {
+                unimplemented!()
+            }
+
+            #[inline]
+            unsafe fn clear(_: *mut $ffi_name) {
+                unimplemented!()
+            }
+        }
+    };
+
+    (@memory_manager_impl $name:ident, $ffi_name:path, @copy $copy_arg:ident $copy_expr:expr, @free $free_arg:ident $free_expr:expr,
+         @init $init_arg:ident $init_expr:expr, @clear $clear_arg:ident $clear_expr:expr) => {
+        #[doc(hidden)]
+        pub struct MemoryManager;
+
+        impl $crate::boxed::BoxedMemoryManager<$ffi_name> for MemoryManager {
+            #[inline]
+            unsafe fn copy($copy_arg: *const $ffi_name) -> *mut $ffi_name {
+                $copy_expr
+            }
+
+            #[inline]
+            unsafe fn free($free_arg: *mut $ffi_name) {
+                $free_expr
+            }
+
+            #[inline]
+            unsafe fn init($init_arg: *mut $ffi_name) {
+                $init_expr
+            }
+
+            #[inline]
+            unsafe fn clear($clear_arg: *mut $ffi_name) {
+                $clear_expr
+            }
+        }
+
+        #[doc(hidden)]
+        impl $crate::translate::Uninitialized for $name {
+            #[inline]
+            unsafe fn uninitialized() -> Self {
+                $name($crate::boxed::Boxed::uninitialized())
+            }
+        }
+    };
 }
 
 enum AnyBox<T> {
@@ -266,15 +336,9 @@ impl<T> fmt::Debug for AnyBox<T> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         use self::AnyBox::*;
         match *self {
-            Native(ref b) => f.debug_tuple("Native")
-                                .field(&(&**b as *const T))
-                                .finish(),
-            ForeignOwned(ptr) => f.debug_tuple("ForeignOwned")
-                                    .field(&ptr)
-                                    .finish(),
-            ForeignBorrowed(ptr) => f.debug_tuple("ForeignBorrowed")
-                                        .field(&ptr)
-                                        .finish(),
+            Native(ref b) => f.debug_tuple("Native").field(&(&**b as *const T)).finish(),
+            ForeignOwned(ptr) => f.debug_tuple("ForeignOwned").field(&ptr).finish(),
+            ForeignBorrowed(ptr) => f.debug_tuple("ForeignBorrowed").field(&ptr).finish(),
         }
     }
 }
@@ -285,6 +349,10 @@ pub trait BoxedMemoryManager<T>: 'static {
     unsafe fn copy(ptr: *const T) -> *mut T;
     /// Frees the object.
     unsafe fn free(ptr: *mut T);
+    /// Initializes an already allocated object.
+    unsafe fn init(ptr: *mut T);
+    /// Clears and frees all memory of the object, but not the object itself.
+    unsafe fn clear(ptr: *mut T);
 }
 
 /// Encapsulates memory management logic for boxed types.
@@ -293,21 +361,16 @@ pub struct Boxed<T: 'static, MM: BoxedMemoryManager<T>> {
     _dummy: PhantomData<MM>,
 }
 
-impl<T: 'static, MM: BoxedMemoryManager<T>> Boxed<T, MM> {
-    #[inline]
-    pub unsafe fn uninitialized() -> Self {
-        Boxed {
-            inner: AnyBox::Native(Box::new(mem::uninitialized())),
-            _dummy: PhantomData,
-        }
-    }
-}
-
 impl<T: 'static, MM: BoxedMemoryManager<T>> Uninitialized for Boxed<T, MM> {
     #[inline]
     unsafe fn uninitialized() -> Self {
         Boxed {
-            inner: AnyBox::Native(Box::new(mem::uninitialized())),
+            inner: {
+                let mut inner = Box::<T>::new(mem::zeroed());
+                MM::init(&mut *inner);
+
+                AnyBox::Native(inner)
+            },
             _dummy: PhantomData,
         }
     }
@@ -395,8 +458,14 @@ impl<T: 'static, MM: BoxedMemoryManager<T>> Drop for Boxed<T, MM> {
     #[inline]
     fn drop(&mut self) {
         unsafe {
-            if let AnyBox::ForeignOwned(ptr) = self.inner {
-                MM::free(ptr.as_ptr());
+            match self.inner {
+                AnyBox::ForeignOwned(ptr) => {
+                    MM::free(ptr.as_ptr());
+                }
+                AnyBox::Native(ref mut box_) => {
+                    MM::clear(&mut **box_);
+                }
+                _ => (),
             }
         }
     }
@@ -404,9 +473,7 @@ impl<T: 'static, MM: BoxedMemoryManager<T>> Drop for Boxed<T, MM> {
 
 impl<T: 'static, MM: BoxedMemoryManager<T>> fmt::Debug for Boxed<T, MM> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        f.debug_struct("Boxed")
-            .field("inner", &self.inner)
-            .finish()
+        f.debug_struct("Boxed").field("inner", &self.inner).finish()
     }
 }
 
@@ -431,7 +498,10 @@ impl<T, MM: BoxedMemoryManager<T>> PartialEq for Boxed<T, MM> {
 impl<T, MM: BoxedMemoryManager<T>> Eq for Boxed<T, MM> {}
 
 impl<T, MM: BoxedMemoryManager<T>> Hash for Boxed<T, MM> {
-    fn hash<H>(&self, state: &mut H) where H: Hasher {
+    fn hash<H>(&self, state: &mut H)
+    where
+        H: Hasher,
+    {
         self.to_glib_none().0.hash(state)
     }
 }
@@ -439,9 +509,7 @@ impl<T, MM: BoxedMemoryManager<T>> Hash for Boxed<T, MM> {
 impl<T: 'static, MM: BoxedMemoryManager<T>> Clone for Boxed<T, MM> {
     #[inline]
     fn clone(&self) -> Self {
-        unsafe {
-            from_glib_none(self.to_glib_none().0 as *mut T)
-        }
+        unsafe { from_glib_none(self.to_glib_none().0 as *mut T) }
     }
 }
 
